/*
    This class demonstrates use of the SimpleCapi.java class 
    for creating a simple java voice-response application
*/

import java.io.*;
import java.net.*;

class Demo4 extends Thread
{
    int num;
    SimpleCapi capi;

    protected void printThread(String message)
    {
	System.out.println("[ Thread "+num+" ]  "+message);
    }

    public Demo4(int num, SimpleCapi capi) 
    {
	this.num = num;
	this.capi = capi;
	printThread("registered");
    }
 
    public void run()
    {
	try {
	    printThread("started");

	    capi.acceptCalls();
	    boolean running = true;
	    String msisdn;
	    while(running)
	    {
		printThread("waiting for call...");
		while(true)
		{
		    ISDNEvent event = capi.listen(600);
		    if(event.type == capi.CAPI_EVENT_INCOMING)
		    {
			printThread("trying to answer to "+event.msisdn+"...");
			if(capi.answer(event)) {
			    msisdn = event.msisdn;
			    break;
			}
			else {
			    printThread("Couldn't answer. Waiting for call...");
			}
		    } else {
			printThread("Unexpected Event: "+capi.eventToString(event));
		    }
		}
		printThread("Connected to "+msisdn+".");
		int jobid = -10;
	
		//jobid = capi.speak("Be welcome to line 33 of I.V.R.");
		jobid = capi.play("welcome.wav");
		
		while(true)
		{
		    ISDNEvent event = capi.listen(600);
		    if(event.type == capi.CAPI_EVENT_DONE &&
			event.jobid == jobid) {
			printThread("message done playing.");
			jobid = capi.play("welcome.wav");
		    } else  if (event.type == capi.CAPI_EVENT_HANGUP) {
			printThread("user hung up...");
			break;
		    } else if (event.type == capi.CAPI_EVENT_DIALTONE) {
			printThread("user pressed the butten: " + event.tone);
			capi.stop(jobid);
		    } else {
			printThread("Unexpected Event: "+capi.eventToString(event));
		    }
		}
		printThread("No longer connected to "+msisdn+".");
	    }
	    capi.close(); 
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public static void main(String args[]) throws Exception
    {
	/* controller 1 */
	SimpleCapi capi1=new SimpleCapi("127.0.0.1", 7347);
	capi1.setMSISDN("33");
	Demo4 thread1 = new Demo4(1, capi1);
	SimpleCapi capi2=new SimpleCapi("127.0.0.1", 7347);
	capi2.setMSISDN("33");
	Demo4 thread2 = new Demo4(2, capi2);

	/* controller 2 */
	SimpleCapi capi3=new SimpleCapi("127.0.0.1", 7347);
	capi3.setMSISDN("32");
	Demo4 thread3 = new Demo4(3, capi3);
	SimpleCapi capi4=new SimpleCapi("127.0.0.1", 7347);
	capi4.setMSISDN("32");
	Demo4 thread4 = new Demo4(4, capi4);

	thread1.start();
	thread2.start();
	thread3.start();
	thread4.start();
    }
}
