/* Festival.cpp

Implementation of Festival.h

Part of the caiviar package. 

Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>
Copyright (c) 2002 MobileX AG, http://www.mobilexag.de

This file is distributed under the GPL, see file COPYING for details. */

#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_CONFIG_H
#undef HAVE_CONFIG_H
#include <festival/festival.h>
#define HAVE_CONFIG_H (1)
#else
#include <festival/festival.h>
#endif
#include "../src/Log.h"
#include "Festival.h"
	
int Festival::instances = 0;

int Festival::initialize(Logger*log)
{
    return 1;
}

void Festival::finalize(Logger*log)
{
}

void Festival::text2Stream (const char *text, unsigned char **outbuffer, unsigned long *size)
{
    log->logf("<notice> Festival::text2Stream(%s)", text);
    EST_Wave wave;
    char filename[256];
    sprintf(filename, "wave%d.wav", lrand48());
    festival_text_to_wave(EST_String(text), wave);
    wave.save(EST_String(filename), EST_String("wav"));
    FILE*fi = fopen(filename, "rb");
    int l;
    if(!fi) {
	log->logf("<error> Festival TTS failed");
	*outbuffer = 0;
	*size = 0;
	return;
    }
    fseek(fi, 0, SEEK_END);
    l = ftell(fi);
    fseek(fi, 0, SEEK_SET);
    if(!l) {
	log->logf("<error> Festival TTS generated no output");
	*outbuffer = 0;
	*size = 0;
	return;
    }

    *outbuffer = (unsigned char*)malloc(l);
    *size = l;
    fread(*outbuffer, l, 1, fi);
    fclose(fi);
    unlink(filename);
}

void Festival::text2File (const char *text, const char *filename)
{
    EST_Wave wave;
    festival_text_to_wave(EST_String(text), wave);
    wave.save(EST_String(text), EST_String(filename));
}

void Festival::text2Audio (const char *text)
{
    festival_say_text(EST_String(text));
}

void Festival::setParams (const char *key, const char* value)
{
    /* test which parameters are for us */
    if(!strncmp(key, "festival.", 9))
    {
	const char*key2 = &key[9];
	log->logf("<warning> Parameter %s not known\n", key2);
    }
}

Festival::Festival(Logger *log)
    :TextToSpeech(log)
{
    if(!Festival::instances) {
	log->logf("<notice> Initializing Festival");
	festival_initialize(1,TTS_OUTPUT_BUFFER);
	festival_init_lang(EST_String("english"));
    }
    Festival::instances++;
}

Festival::~Festival()
{
    //Festival::instances--;
    if(!Festival::instances) {
	log->logf("<notice> Closing Festival");
	festival_tidy_up();
    }
}

