/* TextToSpeech.cpp

Implementation of TextToSpeech.h.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Matthias Kramm <kramm@mobilexag.de>

This file is distributed under the GPL, see file COPYING for details. */

#include "TextToSpeech.h"
#include "../config.h"
#ifdef HAVE_MSAPI
#include "MSSapi.cpp"
#endif
#ifdef HAVE_REALSPEAK
#include "RealSpeak.cpp"
#endif
#ifdef HAVE_FESTIVAL
#include "Festival.cpp"
#endif

#ifdef HAVE_REALSPEAK
# define TTS_CLASS RealSpeak
#else
# ifdef HAVE_MSAPI
#  define TTS_CLASS MSSapi
# else
#  ifdef HAVE_FESTIVAL
#   define TTS_CLASS Festival
#  else
	// no TTS linked
#  endif
# endif
#endif

int TextToSpeech::initialize(Logger*log)
{
#ifdef TTS_CLASS
    return TTS_CLASS::initialize(log);
#else
    return 0;
#endif
}

void TextToSpeech::finalize(Logger*log)
{
#ifdef TTS_CLASS
    TTS_CLASS::finalize(log);
#endif
}

TextToSpeech* TextToSpeech::getImplementation(Logger*log)
{
#ifdef HAVE_REALSPEAK
    return new RealSpeak(log, ".\\Engine");
#else
# ifdef HAVE_MSAPI
	return new MSSapi(log);
# else
#  ifdef HAVE_FESTIVAL
	return new Festival(log);
#  else
	return new TextToSpeech(log);
#  endif
# endif
#endif
}

void TextToSpeech::setParams (const char *key, const char *value)
{
    log->logf("<warning> No Text to Speech engine active");
    return;
}
void TextToSpeech::text2Stream (const char *text, unsigned char **outbuffer, unsigned long *size)
{
    *outbuffer = 0;
    *size = 0;
    log->logf("<warning> No Text to Speech engine active");
    return;
}
void TextToSpeech::text2File (const char *text, const char *filename)
{
    log->logf("<warning> No Text to Speech engine active");
    return;
}
void TextToSpeech::text2Audio (const char *text)
{
    log->logf("<warning> No Text to Speech engine active");
    return;
}

TextToSpeech::TextToSpeech(Logger *log)
{
    this->log = log;
    return;
}

TextToSpeech::~TextToSpeech()
{
    return;
}
