/* Log.cpp

generic Logging class for file, screen and network logging.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#include "../config.h"
#if defined(LINUX) || defined(CYGWIN)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#else
#include <string.h>
#include <winsock2.h>
#include <stdlib.h>
#include <malloc.h>
#include <stdio.h>
#include <time.h>
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif
#include "os.h"


#include "Log.h"

class Log_Internal
{
    public:
    int screenloglevel;
    int fileloglevel;
    int serverloglevel;
    FILE *logFile;
    Mutex log_mutex;
#if defined(LINUX) || defined(CYGWIN)
    int logSocket;
#elif defined(WIN32)
    SOCKET logSocket;
#endif
    bool bLogToSock;
    Log_Internal()
    {
	screenloglevel = fileloglevel = serverloglevel = -1;
	logFile = 0;
	logSocket = 0;
	bLogToSock = 0;
	initMutex(&this->log_mutex);
    }
    void exitLog();
    void initlogSocket(char* servAddr, char* logPort);
    void log(char* logString);
};

void Logger::initLog(char* pLogName, int fileloglevel, char* servAddr, char* logPort, int socketloglevel, int screenloglevel) 
{
    setScreenLogging(screenloglevel);
    setSocketLogging(servAddr, logPort, socketloglevel);
    setFileLogging(pLogName, fileloglevel);
}

Logger::Logger()
{
    this->internal = (void*)new Log_Internal;
    this->initLog(0,-1,0,0,-1,-1);
}

Logger::Logger(int screenloglevel)
{
    this->internal = (void*)new Log_Internal;
    this->initLog(0,-1,0,0,-1,screenloglevel);
}

void Logger::setScreenLogging(int screenloglevel)
{
    ((Log_Internal*)this->internal)->screenloglevel = screenloglevel;
}

void Logger::setFileLogging(char* filename, int fileloglevel)
{
    ((Log_Internal*)this->internal)->fileloglevel = fileloglevel;
    ((Log_Internal*)this->internal)->logFile = NULL;
    if(filename) {
	char*mode = "a+";
	if(filename[0]=='>') {
	    filename++;
	    mode = "w";
	    if(filename[0]=='>') {
		filename++;
		mode = "a+";
	    }
	}

	((Log_Internal*)this->internal)->logFile = fopen(filename, mode);
	if(!((Log_Internal*)this->internal)->logFile) {
	    printf("Couldn't open logfile %s\n", filename);
	    perror(filename);
	}
    }
}

void Logger::setSocketLogging(char* host, char* port, int socketloglevel)
{
    ((Log_Internal*)this->internal)->serverloglevel = socketloglevel;
    ((Log_Internal*)this->internal)->bLogToSock = (host && port && socketloglevel>=0);
    if(((Log_Internal*)this->internal)->bLogToSock)
        ((Log_Internal*)this->internal)->initlogSocket(host, port);
}

void Log_Internal::initlogSocket(char* servAddr, char* logPort) 
{
#if defined(LINUX) || defined(CYGWIN)
    bLogToSock = 0;
#else
// init winsock
    // check and prepare WinSock DLL
    WORD wVersionRequested = MAKEWORD( 2, 2 );
    WSADATA wsaData;
    if ( WSAStartup(wVersionRequested, &wsaData) != 0 ) {
        bLogToSock = false;
        return;
    }
    // Confirm that the WinSock DLL supports 2.2.
    // Note that if the DLL supports versions greater
    // than 2.2 in addition to 2.2, it will still return
    // 2.2 in wVersion since that is the version we
    // requested.

    if ( LOBYTE( wsaData.wVersion ) != 2 || HIBYTE( wsaData.wVersion ) != 2 ) {
        bLogToSock = false;
        return;
    }

    struct hostent *hp;
    hp = gethostbyname(servAddr);
    if (hp == NULL) // we don't know who this host is
    {
        bLogToSock = false;
        return;
    }

    // connect socket
    sockaddr_in SocketAddress;

    memset(&SocketAddress, 0, sizeof(SocketAddress));
    memcpy((char*)&SocketAddress.sin_addr, hp->h_addr, hp->h_length); // set address
    SocketAddress.sin_family = hp->h_addrtype;
    SocketAddress.sin_port = htons((u_short)atoi(logPort));

    logSocket = socket(hp->h_addrtype, SOCK_STREAM, 0);
    if (logSocket == INVALID_SOCKET) {
        bLogToSock = false;
        return;
    }

    // try to connect to the specified socket
    if ( connect(logSocket, (struct sockaddr*)&SocketAddress, sizeof (SocketAddress)) == SOCKET_ERROR) {
        bLogToSock = false;
        return;
    }
    bLogToSock = true;
#endif
}

void Log_Internal::exitLog() {
    // close socket communication
    if(bLogToSock)
#if defined(LINUX) || defined(CYGWIN)
        close(logSocket);
#else
        closesocket(logSocket);
#endif
    // close file
    if(logFile != NULL)
        fclose(logFile);
    destroyMutex(&this->log_mutex);
}

Logger::~Logger() {
    ((Log_Internal*)this->internal)->exitLog();
    delete ((Log_Internal*)this->internal);
}

static char * logimportance[]= {"Fatal","Error","Warning","Notice","Verbose","Debug","Trace"};
static int loglevels=7;
static char * logimportance2[]= {"       ","FATAL  ", "ERROR  ", "WARNING", 
				 "NOTICE ","VERBOSE", "DEBUG  ", "TRACE  "};
void Log_Internal::log(char* logString) 
{
    lockMutex(&this->log_mutex);
    char timebuffer[32];
    char* logBuffer;
    logBuffer = (char*)malloc (strlen(logString) + 24 + 15);
#if defined(LINUX) || defined(CYGWIN)
    {
	time_t t = time(0);
	char* a = ctime(&t);
	int l = strlen(a);
	while(a[l-1] == 13 || a[l-1] == 10)
	    l--;
	a[l]=0;
	sprintf(timebuffer, "%s", a);
    }
#else
char dbuffer[9];
    char tbuffer[9];
    _strdate( dbuffer );
    _strtime( tbuffer );
    sprintf(timebuffer, "%s - %s",dbuffer,tbuffer);
#endif

    // search for <level> field
    int level=-1;
    char*lt=strchr(logString, '<');
    char*gt=strchr(logString, '>');
    if(lt && gt && lt<gt) {
        int t;
        for(t=0;t<loglevels;t++) {
#if defined(LINUX) || defined(CYGWIN)
            if(!strncasecmp(lt+1,logimportance[t],strlen(logimportance[t])))
#else
            if(!strnicmp(lt+1,logimportance[t],strlen(logimportance[t])))
#endif
            {
                logString = gt+1;
                while(logString[0]==' ')
                    logString ++;
                level = t;
                break;
            }
        }
    }

    sprintf(logBuffer, "%s: %s %s", timebuffer, logimportance2[level + 1],logString);

    // we always do exactly one newline.

    int l=strlen(logBuffer)-1;
    while((logBuffer[l]==13 || logBuffer[l]==10) && l>=0) {
        logBuffer[l]=0;
        l--;
    }

    if (level <= screenloglevel) {
        printf("%s\n",logBuffer);
        fflush(stdout);
    }

    if (level <= fileloglevel) {
        if (logFile != NULL) {
            fprintf(logFile, "%s\n", logBuffer);
            fflush(logFile);
        }
    }

    if (level <= serverloglevel) {
        if (bLogToSock) {
            // send data
#if defined(LINUX) || defined(CYGWIN)
            write(logSocket, logBuffer, strlen(logBuffer));
#else
	    send(logSocket, logBuffer, strlen(logBuffer), 0);
#endif
        }
    }
    free (logBuffer);
    unlockMutex(&this->log_mutex);
}

void Logger::log(char* logString) 
{
    ((Log_Internal*)this->internal)->log(logString);
}

void Logger::logf(const char* pszFormat, ...) 
{
    char buf[1024];
    va_list arglist;
    va_start(arglist, pszFormat);
    vsprintf(buf, pszFormat, arglist);
    va_end(arglist);
    log(buf);
}

