/* os.h

header file for operating system dependent functions

Part of the caiviar package. 

Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#ifndef __os_h__
#define __os_h__
#include "../config.h"

#if defined(LINUX) || defined(CYGWIN)
#include <unistd.h>
#include <pthread.h>
#endif

/*
os-dependent headers should one day only be included in this file

#if defined(LINUX) || defined(CYGWIN)
#include <pthread.h>
#include <unistd.h>
#include <signal.h>
#else
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#endif*/

/* thread_t structure */

struct Thread 
{
#if defined(WIN32)
    DWORD threadID;
    HANDLE thread;
#elif defined(LINUX) || defined(CYGWIN)
    pthread_t thread;
#endif
};

#if defined(WIN32)
#define THREAD_FUNCTION DWORD WINAPI
#elif defined(LINUX) || defined(CYGWIN)
#define THREAD_FUNCTION void*
#endif

/* THREAD_CREATE */

#ifdef WIN32
#define createThread(str,function,data) \
{\
    (str)->thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)(function), (data), 0, (LPDWORD)&((str)->threadID));\
}
#elif defined(LINUX) || defined(CYGWIN)
#define createThread(str,function,data) \
{\
    pthread_create(&((str)->thread), 0, (function), (data));\
}
#endif

/* Sleep(seconds) */

/* todo: is the Win32 API Sleep 1/1000th seconds?*/
#if defined(LINUX) || defined(CYGWIN)
#define Sleep(a) usleep(a*1000)
#endif

/* strcasecmp */
#ifdef WIN32
#define strcasecmp stricmp
#endif

/* InitMutex */

#if defined(WIN32)
#define initMutex(mutex) {InitializeCriticalSection(mutex);}
#else
#define initMutex(mutex) {pthread_mutex_init(mutex, 0);}
#endif

/* DestroyMutex */

#if defined(WIN32)
#define destroyMutex(mutex) {DeleteCriticalSection(mutex);}
#else
#define destroyMutex(mutex) {pthread_mutex_destroy((mutex));}
#endif

/* LockMutex */

#if defined(LINUX) || defined(CYGWIN)
#define lockMutex(mutex) pthread_mutex_lock(mutex)
#define unlockMutex(mutex) pthread_mutex_unlock(mutex)
#elif defined(WIN32)
#define lockMutex(mutex) EnterCriticalSection(mutex)
#define unlockMutex(mutex) LeaveCriticalSection(mutex)
#endif

/* Mutex */

#if defined(LINUX) || defined(CYGWIN)
#define Mutex pthread_mutex_t
#elif defined(WIN32)
#define Mutex CRITICAL_SECTION
#endif

#endif
